import yfinance as yf
import spacy
from spacytextblob.spacytextblob import SpacyTextBlob
import spacy.cli

from .CONSTANTS import ARTICLE_WEIGHT
from .extract_ticker_headlines import extract_ticker_headlines
from .get_full_article_text import get_full_article_text
from .get_yf_ticker_news import get_yf_ticker_news


def analyze_stock_headlines(ticker: str) -> float:
    r"""
    Perform sentiment analysis on the headlines of a ticker.
    """
    try:
        nlp = spacy.load("en_core_web_lg")
        nlp.add_pipe("spacytextblob")
    except:
        spacy.cli.download("en_core_web_lg")
        nlp = spacy.load("en_core_web_lg")
        nlp.add_pipe("spacytextblob")

    yf_ticker_news = get_yf_ticker_news(ticker)

    headlines = extract_ticker_headlines(yf_ticker_news)
    article_texts = [
        get_full_article_text(item["link"]) for item in yf_ticker_news
    ]
    headline_sentiment: float = 0.0
    article_sentiment: float = 0.0
    for headline in headlines:
        doc = nlp(headline)
        headline_sentiment += doc._.blob.polarity

    for text in article_texts:
        doc = nlp(text or "")
        article_sentiment += doc._.blob.polarity

    if len(headlines) > 0:
        headline_sentiment = headline_sentiment / len(headlines)

    if len(article_texts) > 0:
        article_sentiment = article_sentiment / len(article_texts)

    return (
        ARTICLE_WEIGHT * article_sentiment
        + (1 - ARTICLE_WEIGHT) * headline_sentiment
    )



ARTICLE_WEIGHT = 0.75


FOREX_TICKERS = [
    "EURUSD=X",
    "JPY=X",
    "GBPUSD=X",
    "EURJPY=X",
    "EURGBP=X",
    "AUDUSD=X",
    "NZDUSD=X",
    "GBPJPY=X",
    "EURCAD=X",
    "EURSEK=X",
    "EURCHF=X",
    "EURHUF=X",
    "EURJPY=X",
    "CNY=X",
    "HKD=X",
    "SGD=X",
    "INR=X",
    "MXN=X",
    "PHP=X",
    "THB=X",
    "ZAR=X",
    "RUB=X",
]



import argparse
from speech_user_interface import (
    speech_user_interface,
    speak_text,
    compare_strings,
)

from .analyze_stock_headlines import analyze_stock_headlines
from .create_ticker_list import create_ticker_list
from .CONSTANTS import FOREX_TICKERS

__name__ == "__main__"
__all__ = ["train", "forecast", "saturn_utils", "saturn_forecaster"]


def analyze_parse_args():
    parser = argparse.ArgumentParser(
        description="The analyze method of Chronos."
    )
    parser.add_argument(
        "--ticker",
        type=str,
        help="Call with --ticker to pass the desired ticker to analyze",
        required=True,
        default=False,
        const=True,
        nargs="?",
        dest="ticker",
    )
    return parser.parse_args()


def function_to_run(input_text: str):
    if compare_strings(input_text, "scan"):
        scan()


def main():
    # build()
    speech_user_interface()
    analyze()


def analyze():
    print("Analyzing...")

    args = analyze_parse_args()

    return analyze_stock_headlines(args.ticker)


def scan(forex: bool = True):
    print("Scanning for forex currency pairs to buy/short...")
    speak_text("Scanning for forex currency pairs to buy/short...")

    largest_sentiment = 0.0
    lowest_sentiment = 10000.0
    if not forex:
        tickers = create_ticker_list()
    else:
        tickers = FOREX_TICKERS
    best_ticker: str = tickers[0]
    worst_ticker: str = tickers[0]
    i: int = 0
    for ticker in tickers:
        sentiment = analyze_stock_headlines(ticker)

        if sentiment > largest_sentiment:
            best_ticker = ticker
            largest_sentiment = sentiment
        if sentiment < lowest_sentiment:
            worst_ticker = ticker
            lowest_sentiment = sentiment

        if i % 10 == 0:
            print(
                "(best_ticker, largest_sentiment), (worst_ticker, lowest_sentiment):",
                (best_ticker, largest_sentiment),
                (worst_ticker, lowest_sentiment),
            )
            speak_text(
                "(best_ticker, largest_sentiment), (worst_ticker, lowest_sentiment):"
                + f"{(best_ticker, largest_sentiment)}"
                + f"{(worst_ticker, lowest_sentiment)}"
            )

        i += 1

    print(
        "(best_ticker, largest_sentiment), (worst_ticker, lowest_sentiment):",
        (best_ticker, largest_sentiment),
        (worst_ticker, lowest_sentiment),
    )
    speak_text(
        "(best_ticker, largest_sentiment), (worst_ticker, lowest_sentiment):"
        + f"{(best_ticker, largest_sentiment)}"
        + f"{(worst_ticker, lowest_sentiment)}"
    )
    return (best_ticker, largest_sentiment), (worst_ticker, lowest_sentiment)



from bs4 import BeautifulSoup
import requests

headers = {
    "accept": "*/*",
    "User-Agent": "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_15_7) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/101.0.4951.64 Safari/537.36 Edg/101.0.1210.53",
    "Accept-Language": "en-US,en;q=0.9,it;q=0.8,es;q=0.7",
    "referer": "https://www.google.com/",
    "cookie": "DSID=AAO-7r4OSkS76zbHUkiOpnI0kk-X19BLDFF53G8gbnd21VZV2iehu-w_2v14cxvRvrkd_NjIdBWX7wUiQ66f-D8kOkTKD1BhLVlqrFAaqDP3LodRK2I0NfrObmhV9HsedGE7-mQeJpwJifSxdchqf524IMh9piBflGqP0Lg0_xjGmLKEQ0F4Na6THgC06VhtUG5infEdqMQ9otlJENe3PmOQTC_UeTH5DnENYwWC8KXs-M4fWmDADmG414V0_X0TfjrYu01nDH2Dcf3TIOFbRDb993g8nOCswLMi92LwjoqhYnFdf1jzgK0",
}


def get_full_article_text(yfinance_article_url: str):
    try:
        return get_body_text(yfinance_article_url)
    except:
        return get_readmore_link(yfinance_article_url)


def get_body_text(yfinance_article_url: str):
    html_response = requests.get(yfinance_article_url, headers=headers)
    soup = BeautifulSoup(html_response.text, features="lxml")
    full_article_text = soup.select("div.caas-body")[0].text

    return full_article_text


def get_readmore_link(yfinance_article_url: str):
    print("yfinance_article_url:", yfinance_article_url)
    html_response = requests.get(yfinance_article_url, headers=headers)
    if html_response.status_code == 404:
        return ""

    print("html_response:", html_response)
    soup = BeautifulSoup(html_response.text, features="lxml")
    print("soup:", soup)
    full_article_link = soup.find("Continue Reading")
    print("full_article_link:", full_article_link)

    print("full_article_link:", full_article_link)

    full_article_text = requests.get(full_article_link, headers=headers).text
    print("full_article_text:", full_article_text)

    exit()

    return full_article_text



import pandas as pd


def create_ticker_list():
    return (
        pd.read_csv(f"./chronos_ai/data/nasdaq_screener_1711270178562.csv")
        .dropna()
        .reset_index()
        .loc[:, "Symbol"]
    )
